package RUcoder;

use strict;
use vars qw($VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(&RUcoding &RUdecoding);
$VERSION = '1.00';


sub RUcoding {

	my %h = (@_);
	my $add = int $h{add} || 0;
	my $ANSI = unpack('H*', $h{line}).'0d';

	my $key = genkey( $h{key} );		#closed key
	my $KEY = genKEY();					#open key

	my $path_1 = Coder( genstr( addSymbol($add, $KEY) ), $key );
	my $path_2 = Coder( genstr( addSymbol($add, $ANSI) ), $KEY );

	return $path_1.$path_2;
}


sub RUdecoding {

	my %h = (@_);
	my $add = int $h{add} || 0;
	my $first = sprintf('%.0f', ($add+2)/2) * 16;

	my $path_1 = substr( $h{line}, 0, $first );
	my $path_2 = substr( $h{line}, $first );

	my $key = genkey( $h{key} );												#closed key
	my $KEY = substr( delSymbol( $add, lc Decoder( $path_1, $key ) ), 0,16 );	#open key

	my $ANSI = delSymbol( $add, Decoder( substr($h{line}, $first), $KEY ) );

	my $line = pack('H*', $ANSI);
	$line =~ s/\s[^\s]*$//g;

	return $line;
}

#########################################

sub Coder {		#Rearrangement on a key

	my( $str, $key ) = @_;
	my $len = length $str;
	my( $code, $i )=();

	for $i ( 0..int($len-1)/16 ){

		$code = $code.'0'x16;

		for( 0..15 ){

			my $n = $i*16 + hex substr($key,$_,1);		#Number in a line

			my $s = substr( $str, $n, 1 ) || 0;			#Symbol for replacement

			substr( $code, $i*16+$_, 1 ) = $s;			#Substitution
		}
	}

	return $code;
}


sub Decoder {	#Restoration on a key

	my( $code, $key ) = @_;
	my $len = length $code;
	my( $str, $i )=();

	for $i ( 0..int($len-1)/16 ){

		$str = $str.'0'x16;

		for( 0..15 ){

			my $n = $i*16 + hex substr($key,$_,1);		#Number in a line

			my $s = substr( $code, $i*16+$_, 1 ) || 0;	#Symbol for replacement

			substr( $str, $n, 1 ) = $s;					#Substitution
		}
	}

	return $str;
}


sub genKEY {	#Reception of an open key

	my( $key )=();

	while( length $key<16 ){
		my $n = sprintf('%x', int rand(16));
		$key = $key.$n if $key!~/$n/;
	}

	return $key;
}


sub genkey {	#Reception of the closed key

	my( $pas, $key )=@_;
	$pas = lc $pas;

	for( 0..15 ){
		my $n = substr( $pas, $_, 1 );
		$key=$key.$n if $key!~/$n/ && defined $n && $n=~/[0-9a-f]/;
	}

	for( 0..15 ){
		my $n = sprintf('%x', $_);
		$key = $key.$n if $key!~/$n/;
	}

	return $key;
}


sub genstr {	#Addition of a line up to 16x

	my $len = length($_[0]);
	my $n = 16-sprintf('%.0f', ($len/16 - (int ($len/16)))*16);
	return $_[0] if $n==16;

	my $str = '';
	for( 1..$n ){
		$str = $str.sprintf('%x', (int rand(15))+1) if $n!=16;
	}

	return $_[0].$str;
}


sub addSymbol {

	my( $add, $line )=@_;
	return $line unless $add;

	$add = int $add; my $addD = ($add+2)/2;
	my $len = length $line;
	my $lineN = '0'x(int($len*$addD));
	my $lenD = sprintf('%.0f', $len/2);

	for ( 0..$lenD-1 ){

		my $i = int( $_*$addD*2 );
		substr( $lineN, $i, 2 ) = substr( $line, $_*2, 2 );

		for( 0..$add-1 ){
			substr( $lineN, $i+$_+2, 1 ) = sprintf('%x', int rand(16));
		}
	}

	return $lineN;
}


sub delSymbol {

	my( $add, $line )=@_;
	return $line unless $add;

	$add = int $add; my $addD = ($add+2)/2;
	my $len = length $line;
	my $lineN = '0'x(int($len/$addD));
	my $lenD = sprintf('%.0f', $len/($addD*2));

	for ( 0..$lenD-1 ){

		my $i = int( $_*$addD*2 );
		substr( $lineN, $_*2, 2 ) = substr( $line, $i, 2 );
	}

	return $lineN;
}

1;
__END__

=head1 NAME

RUcoder - Coding and decoding of the information for transfer and reception under the protocol HTTP

=head1 SYNOPSIS

use RUcoder;

$key = '43DF057AC1894BE';	#Key for coding and decoding of the information

$line = 'Test line for check:)';	#The information for coding and transfer on HTTP

## Coding
$code = RUcoding( key=>$pas, line=>$line );	#Code for transfer on HTTP

## Decoding
$lineN = RUdecoding( key=>$pas, line=>$code );	#The received information on HTTP ($line eq $lineN)

exit();

=head1 DESCRIPTION

The module codes and decodes given for transfer under the protocol HTTP.
With the help of the Module RUcoder it is possible to transfer and to accept the symbolical data of the protocol ANSI.

A Simple Example. Given for coding:
$line = 'Test line for check:)';

1. Task of a key and Coding of the data.
We set a key:
$key = '43DF057AC1894BE';
In a key can be 3..16 symbols (0..9A..F)

We code a line:
$code = RUcoding( key=>$key, line=>$line );

We receive a line of such kind:
d571a8e236c4f9b06663925767e0c4542626370666820f65106b923316f95a65 ($code)

We transfer a line on HTTP.
We receive a line:
$lineN = RUdecoding( key=>$key, line=>$code );

The lines should be equal: $line eq $lineN

=head1 AUTHOR

V. V. Gorhilin, arhitrade@mail.ru
http://uniq-ip.biz/gorchilin/RUcoder/index.htm
2004

=cut
